#************************************************************ 
#
# Component       CNF  
#
# Synopsis        Node State Manager
#
#
# Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
# Use is subject to license terms.
#
#
#************************************************************
#
# #ident	"@(#)NSM.pm	1.2	05/10/03 SMI"
#
#************************************************************

#=head1 External Access (Node State Manager)
#
#=over 4
#
#=cut

package NSM ;

use strict;
use VAR;
use LOG ;
use COMMON;

#--------------------------------------------------------------
#
#  check_dir
#
#--------------------------------------------------------------

sub check_dir {

  my ($dirname, $status_msg) = @_;


  my $status = "OK";
  if (! -d $dirname) {
    $status = "FAILED";

#=item directory <dirname> does not exist
#
#    One of the directories used by the Node State Manager does not exist.
#    Create the mentionned directory.
#
#=cut

    LOG::printError("directory %s does not exist", $dirname);
  }

  LOG::printResult($status, $status_msg);

}

#--------------------------------------------------------------
#
#  check_interface
#
#--------------------------------------------------------------

sub check_interface  {

  my $status = "OK";
  my $physical_interface;
  if (! COMMON::exist_physical_interface
      ($E_variables{NSM_EXTERNAL_MASTER_NIC}, \$physical_interface)) {

#=item physical interface <nic> for external access does not exist
#
#    The specified interface does not exist. If you have configured an alias
#    only the physical interface is tested (the alias is dynamically created).
#
#=cut

    LOG::printError("physical interface %s for external access does not exist", $physical_interface);
    $status = "FAILED";
  }

  LOG::printResult($status, "Physical interface for external access");
	     
}

#--------------------------------------------------------------
#
#  check_configuration
#
#--------------------------------------------------------------

sub check_configuration  {

  if (OS::is_feature_installed("NSM") ne "TRUE") {
    return ;
  }

  LOG::printMsg(0, "") ;
  LOG::printMsg(0, "Node State Manager checking") ;
  LOG::printMsg(0, "") ;


  check_dir($E_variables{NSM_MASTER_DIR}, "Existence of the master script directory");
  check_dir($E_variables{NSM_VICEMASTER_DIR}, "Existence of the vice-master script directory");

  check_dir($E_variables{NSM_LOG_MASTER_DIR}, "Existence of the master log directory");
  check_dir($E_variables{NSM_LOG_VICEMASTER_DIR}, "Existence of the vice-master log directory");
  

  check_interface();
}

#--------------------------------------------------------------
#
#  main
#
#--------------------------------------------------------------

{
}
