#************************************************************ 
#
# Component      CNF  
#
# Synopsis       Operating System and Packages
#
#
# Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
# Use is subject to license terms.
#
#
#************************************************************
#
#ident	"@(#)OS.pm	1.34	05/09/13 SMI"
#
#************************************************************

#=head1 Operating System and Packages
#
#    This section also contains descriptions for error than can be detected
#    in circumstances that are not related to installation.
#
#=over 4
#
#=cut

package OS ;

use strict;
use VAR;
use LOG;

my $G_category = "OS" ;

my @G_empty_array = ();

#
#  Packages list
#
#  0: name of the package
#  1: local (LOCAL), shared (SHARED)
#  2: version or list of versions (when EXACT is tested) separated by a comma
#  3: on master-eligible (YES or NO)
#  4: on diskless (YES or NO)
#  5: on dataless (YES or NO)
#  6 check exactly one of the listed version (EXACT) or at least the specific version (AT_LEAST)
#  7: mandatory package
#  8: feature which the package is part of

my $G_package_name      = 0 ;
my $G_package_location  = 1 ;
my $G_package_version   = 2 ;
my $G_package_men       = 3 ;
my $G_package_diskless  = 4 ;
my $G_package_dataless  = 5 ;
my $G_package_comp      = 6 ;
my $G_package_mandatory = 7 ;
my $G_package_feature   = 8 ;

my @G_packages_S8U7B8 = (
		  # CGTP packages
		  ["SUNWnhtp8", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		  ["SUNWnhtu8", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		 ) ;

my @G_packages_S8PSR1 = (
		  # CGTP packages
		  ["SUNWnhtp8", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		  ["SUNWnhtu8", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		 ) ;

my @G_packages_S8PSR3 = (
		  # CGTP packages
		  ["SUNWnhtp8", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		  ["SUNWnhtu8", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		 ) ;

my @G_packages_S9HWPL3 = (
		  # CGTP packages
		  ["SUNWnhtp9", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		  ["SUNWnhtu9", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		    ) ;

my @G_packages_S9U1 = (
		  # CGTP packages
		  ["SUNWnhtp9", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		  ["SUNWnhtu9", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		    ) ;

my @G_packages_S9_GT_U5 = (
		  # CGTP packages
		  ["SUNWnhtp9", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		  ["SUNWnhtu9", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CGTP" ] ,
		    ) ;

my @G_packages_common = (	 
		  # ADM packages
		  ["SUNWnhadm", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "ADM" ] ,

		  # CMM packages
		  ["SUNWnhhb",  "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "BASE" ] ,
		  ["SUNWnhcdt", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "BASE" ] ,
		  ["SUNWnhcmd", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CMM" ] ,
		  ["SUNWnhcma", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CMM" ] ,
		  ["SUNWnhcmb", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "CMM" ] ,

		  # SNDR packages
		  ["SUNWscmr",  "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,
		  ["SUNWscmu",  "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,
		  ["SUNWspsvr", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,
		  ["SUNWspsvu", "LOCAL",       "2.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,
		  ["SUNWrdcr",  "LOCAL",       "2.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,
		  ["SUNWrdcu",  "LOCAL",       "2.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,

		  # RNFS packages
		  ["SUNWnhfsa", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,
		  ["SUNWnhfsb", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "RNFS" ] ,

		  # PMD packages
		  ["SUNWnhpma", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "PMD" ] ,
		  ["SUNWnhpmb", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "PMD" ] ,
		  ["SUNWnhpmm", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "PMD" ] ,
		  ["SUNWnhpmn", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "PMD" ] ,
		  ["SUNWnhpms", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "PMD" ] ,

		  # NSM packages
		  ["SUNWnhnsa", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "NO",  "NSM" ] ,
		  ["SUNWnhnsb", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "NO",  "NSM" ] ,

		  # NLS packages
#		  ["SUNWnhnla", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "NO",  "NLS" ] ,
#		  ["SUNWnhnlb", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "NO",  "NLS" ] ,

		  # JDK packages
		  ["SUNWj3rt",  "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "JDK" ] ,

		  # MA and JDMK packages
		  ["SUNWnhmas", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "YES", "NMA" ] ,
		  ["SUNWjsnmp", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "NMA" ] ,
		  ["SUNWjdrt",  "SHARED",      "4.2",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "NMA" ] ,
		  ["SUNWnhmaj", "SHARED",      "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "NMA" ] ,
		  ["SUNWnhmal", "SHARED",      "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "NMA" ] ,
		  ["SUNWnhmad", "SHARED",      "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "YES", "NMA" ] ,

		  # RBS packages
		  ["SUNWnhrbb", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "NO",  "RBS" ] ,
		  ["SUNWnhrbs", "LOCAL",       "1.0",    "YES", "NO",  "NO",  "AT_LEAST", "NO",  "RBS" ] ,

		  # WDT packages
		  ["SUNWnhwdt", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "NO",  "WDT" ] ,

		  # SMCT packages
		  ["SUNWnhsms", "LOCAL",       "1.0",    "YES", "YES", "YES", "AT_LEAST", "NO",  "SMCT" ] ,
			);

# current table
my @G_packages;

#
#  Patch list
#
#  0: name of the patch
#  1: local (LOCAL), shared (SHARED)
#  2: on master-eligible (YES or NO)
#  3: on diskless (YES or NO)
#
# Note that, on Solaris 8, the CGTP patch is dependent on the kernel patch
# not on the Solaris version

my $G_patch_name     = 0 ;
my $G_patch_location = 1 ;
my $G_patch_men      = 2 ;
my $G_patch_diskless = 3 ;
my $G_patch_dataless = 4;

my @G_patches_S8U7B8 = (
		    # Solaris patch
		    ["108727-22", "LOCAL",  "YES", "YES", "YES" ] ,
		    ["111958-02", "LOCAL",  "YES", "NO", "YES" ] ,
		   ) ;

my @G_patches_S8PSR1 = (
		    # Solaris patch
		    ["108727-22", "LOCAL",  "YES", "YES", "YES" ] ,
		   ) ;

my @G_patches_S8PSR3 = (
		   ) ;

my @G_patches_S9HWPL3 = (
		    # Solaris patch
		    ["113318-03", "LOCAL",  "YES", "YES", "YES" ] ,
		    ["112965-01", "LOCAL",  "YES", "YES", "YES" ] ,

		    # CGTP patch
		    ["112233-03", "LOCAL",  "YES", "YES", "YES" ] ,
		    ["112902-06", "LOCAL",  "YES", "YES", "YES" ] ,
		    ["112904-01", "LOCAL",  "YES", "YES", "YES" ] ,
		    ["112917-01", "LOCAL",  "YES", "NO" , "YES" ] ,
		    ["112918-01", "LOCAL",  "YES", "NO" , "YES" ] ,
		    ["112919-01", "LOCAL",  "YES", "NO" , "YES" ] ,
		   ) ;

my @G_patches_S9U1 = (
		    # Solaris patch
		    ["113318-03", "LOCAL",  "YES", "YES", "YES" ] ,

		    # CGTP patch
		    ["112233-03", "LOCAL",  "YES", "YES", "YES" ] ,
#		    ["112902-06", "LOCAL",  "YES", "YES", "YES" ] ,
		     ) ;

my @G_patches_S9_GT_U5 = (
		     ) ;

my @G_patches_common = (
		    # SNDR patch
		    ["116710-02", "LOCAL",  "YES", "NO",  "NO"  ] ,
			);

#
# list of the supported releases
#
my @G_supported_release = 
  (
   { 
    idstring    => "Solaris 8 2/02 s28s_u7wos_08a SPARC",
    pkg_list    => \@G_packages_S8U7B8,
    patch_list  => \@G_patches_S8U7B8,
   } ,
   { 
    idstring    => "Solaris 8 HW 12/02 s28s_hw1wos_06a SPARC",
    pkg_list    => \@G_packages_S8PSR1,
    patch_list  => \@G_patches_S8PSR1,
   } ,
   {
    idstring    => "Solaris 8 HW 7/03 s28s_hw3wos_05a SPARC",
    pkg_list    => \@G_packages_S8PSR3,
    patch_list  => \@G_patches_S8PSR3,
   } ,
   { 
    idstring    => "Solaris 9 s9_58shwpl3 SPARC",
    pkg_list    => \@G_packages_S9HWPL3,
    patch_list  => \@G_patches_S9HWPL3,
   } ,
   { 
    idstring    => "Solaris 9 9/02 s9s_u1wos_08b SPARC",
    pkg_list    => \@G_packages_S9U1,
    patch_list  => \@G_patches_S9U1,
   } ,
   { 
    idstring    => "Solaris 9 4/04 s9s_u6wos_08a SPARC",
    pkg_list    => \@G_packages_S9_GT_U5,
    patch_list  => \@G_patches_S9_GT_U5,
   } ,
   { 
    idstring    => "Solaris 9 9/04 s9s_u7wos_09 SPARC",
    pkg_list    => \@G_packages_S9_GT_U5,
    patch_list  => \@G_patches_S9_GT_U5,
   } ,
);


#
#  Patch list depending on the kernel patch
#
#  0: name of the patch
#  1: local (LOCAL), shared (SHARED)
#  2: on master-eligible (YES or NO)
#  3: on diskless (YES or NO)
#
# Note that, on Solaris 8, the CGTP patch is dependent on the kernel patch
# not on the Solaris version
#
my @G_patches_108528_13 = (
			   # CGTP patch
			   ["112281-02", "LOCAL",  "YES", "YES", "YES" ] ,
			  ) ;

my @G_patches_108528_18 = (		    
			   # CGTP patch
			   ["115419-01", "LOCAL",  "YES", "YES", "YES" ] ,
			  ) ;

my @G_patches_108528_21 = (		    
			   # CGTP patch
			   ["116036-02", "LOCAL",  "YES", "YES", "YES" ] ,
			  ) ;

my @G_patches_108528_22 = (		    
			   # CGTP patch
			   ["116036-02", "LOCAL",  "YES", "YES", "YES" ] ,
			  ) ;

#
# list of the supported kernel patch
#
my @G_supported_kernel_patch = 
  (
   { 
    idstring    => "Generic_108528-13",
    pkg_list    => \@G_empty_array,
    patch_list  => \@G_patches_108528_13,
   } ,
   { 
    idstring    => "Generic_108528-18",
    pkg_list    => \@G_empty_array,
    patch_list  => \@G_patches_108528_18,
   } ,
   { 
    idstring    => "Generic_108528-21",
    pkg_list    => \@G_empty_array,
    patch_list  => \@G_patches_108528_21,
   } ,
   { 
    idstring    => "Generic_108528-22",
    pkg_list    => \@G_empty_array,
    patch_list  => \@G_patches_108528_22,
   } ,
   { 
    # Solaris 9
    idstring    => "Generic",
    pkg_list    => \@G_empty_array,
    patch_list  => \@G_empty_array,
   } ,
);

# current table
my @G_patches;

#
# package acessibility
#
my $G_swdb_local_dir = "/SUNWcgha/local/export/services" ;
my $G_swdb_services = "/SUNWcgha/services" ;
my $G_swdb_mounting_point = "/SUNWcgha/swdb" ;
my $G_swdb_accessible = "FALSE" ;
my $G_swdb_remote_access = "FALSE" ;  
my $G_swdb_dir = "" ;
my $G_swdb_basedir = "" ;
my $G_swdb_path_confshare = "var/sadm/system/admin" ;
my $G_swdb_file_confshare = "INST_RELEASE" ;

#
# installed patch list
#
my @G_local_patch  = ();
my @G_shared_patch = ();
my $G_local_patch_loaded = $FALSE;
my $G_shared_patch_loaded = $FALSE;

#--------------------------------------------------------------
#
# set_os_env
#
#    determine the OS version and set all the related value
#
#--------------------------------------------------------------

sub set_os_env {

  my $release_file = "/etc/release";
  my $idx=-1;

  my $release = qx{/usr/bin/grep "Solaris" $release_file};
  $release =~ s/^[\s\t]*//;
  $release =~ s/[\s\t]*$//;

  # is it a supported release ?
  my $found = $FALSE;
  for my $i (0..$#G_supported_release) {
    if ($G_supported_release[$i]{idstring} eq $release) {
      $idx = $i;
      $found = $TRUE;
    }
  }

  if ($found) {
    my $ref = $G_supported_release[$idx]{pkg_list};
    push @G_packages, @$ref;

    $ref = $G_supported_release[$idx]{patch_list};
    push @G_patches, @$ref;

  } else {

    LOG::printWarning("Unsupported Solaris release (%s): only standard packages and patches will be checked", $release);

  }
  
  # retrieve the kernel patch release
  chomp(my $solaris_release = qx{/usr/bin/uname -r}) ;
  chomp(my $kernel_patch  = qx{/usr/bin/uname -v}) ;

  # is it a supported kernel patch release ?
  # only check on Solaris 8
  if ($solaris_release eq "5.8") {
    $found = $FALSE;
    for my $i (0..$#G_supported_kernel_patch) {
      if ($G_supported_kernel_patch[$i]{idstring} eq $kernel_patch) {
	$idx = $i;
	$found = $TRUE;
      }
    }
    
    if ($found) {
      my $ref = $G_supported_kernel_patch[$idx]{pkg_list};
      push @G_packages, @$ref;
      
      $ref = $G_supported_kernel_patch[$idx]{patch_list};
      push @G_patches, @$ref;
      
    } else {
      
      LOG::printWarning("Unsupported Solaris kernel patch (%s): only standard packages and patches will be checked", $kernel_patch);
      
    }
  }

  push @G_packages, @G_packages_common;
  push @G_patches, @G_patches_common;

}
#--------------------------------------------------------------
#
#  determine_swdb
#
#  shared package info database is
#  . only accessible from one node during installation
#    (when mounting points are not yet defined)
#  . accessible from all noeds after reboot (mounting point
#    has been defined on all nodes)
#
#--------------------------------------------------------------

sub determine_swdb {
  # is the mounting point defined ?
  my $result = qx{/usr/sbin/mount | /usr/bin/grep $G_swdb_mounting_point} ;
  
  if ($result eq "") {
    # no mounting point: check if we are on the main node for installation
    if ( -d $G_swdb_local_dir ) {
      $G_swdb_accessible = "TRUE" ;
      $G_swdb_dir = $G_swdb_local_dir ;
    }
    
  } else {
    
    #
    # determine the base dir
    #
    $result = qx{/usr/sbin/mount | /usr/bin/grep $G_swdb_services};
    if ($result eq "") {
      LOG::printError("Cannot determine the current BASE_DIR for shared packages") ;
      return ;
    }
    # extract the remote sub-directory name as the basedir
    if ($result !~ m{$G_swdb_local_dir/([^\s\t]*)}) {
      LOG::printError("Cannot determine the current BASE_DIR for shared packages") ;
      return ;
    }
    $G_swdb_basedir = "/$1";
    $G_swdb_accessible = "TRUE" ;
    $G_swdb_remote_access = "TRUE" ;
    $G_swdb_dir = $G_swdb_mounting_point ;
  }
  
}

#--------------------------------------------------------------
#
#  guess_node_role
#
#    try to guess the node role by testing the mounting point
#    of the root partition since CMM can't be used
#    and nhfs.conf may be not already configured
#
#--------------------------------------------------------------

sub guess_node_role
  {
    my $result;
    chomp($result = qx{mount -v | /usr/bin/grep " on / type "}) ;
    $result =~ m/ on \/ type ([a-z]*) /;
    if ($1 eq "ufs")
      {
	my $version ;
	my $name ;
	my $basedir ;
	
	my $return = get_package("SUNWnhfsa", "LOCAL", 
				 \$version, \$name, \$basedir) ;
	if ($return eq "OK") {
	  return "DISKFULL" ;
	} else {
	  return "DATALESS"
	}
      }

    return "DISKLESS" ;
  }

#--------------------------------------------------------------
#
#  compare_version
#
#--------------------------------------------------------------

sub compare_version {

  my ($actual, $expected, $option) = @_ ;

  my $version;

  if ($option eq "EXACT")
      {
	foreach $version (split(",",$expected))
	  {
	    if ($actual eq $version)
	      {
		return "TRUE";
	      }
	  }
	return "FALSE" ;
      }
    else
      {
	if ($option eq "AT_LEAST")
	  {
	    my @part_current = split('\.', $actual) ;
	    my @part_expected = split('\.', $expected) ;
	    my $atleast = "TRUE" ;
	    my $end = "FALSE" ;
	    my $exp_size = scalar(@part_expected) ;
	    my $cur_size = scalar(@part_current) ;
	    my $i = 0 ;
	    while (($i < $exp_size) and ($end eq "FALSE"))
	      {
		if ($i < $cur_size)
		  {
		    if ($part_current[$i] > $part_expected[$i])
		      {
			$end = "TRUE" ;
		      }
		    else
		      {
			if ($part_current[$i] < $part_expected[$i])
			  {
			    $atleast = "FALSE" ;
			    $end = "TRUE" ;
			  }
		      }
                  }
		else
		  {
		    $atleast = "FALSE" ;
		    $end = "TRUE" ;
		  }
		$i++ ;
	      }
	      
	    if ($atleast eq "FALSE")
	      {
		return "LESS" ;
	      }
	  }
      }
}

#--------------------------------------------------------------
#
#  get_package
#
#    get info about a specified package
#
#    1 : in: name of the package
#    2 : in: location
#
#    3 : out: version of the package
#    4 : out: complete name
#    5 : out: base directory
#
#    return :
#    . OK if the package exists
#    . NA if the package is not part of this node (Not Applicable)
#    . NOK if the package doesn't exist
#
#--------------------------------------------------------------

sub get_package
  {
    my ($L_package, $L_location, $R_version, $R_name, $R_basedir) = @_ ;

    my $command = "" ;
    my $dir = "" ;

    # basedir may be not present in package information so it should be initialized
    $$R_basedir = "" ; 

    if ($L_location eq "SHARED") {
      $dir = "$G_swdb_dir" ;
      $command = "-R $G_swdb_dir" ;
    } elsif ($L_location ne "LOCAL") {
      LOG::printError("Invalid location (%s) for package %s", $L_location, $L_package) ;
      exit 1;
    }

    my $L_result = system("/usr/bin/pkginfo $command $L_package > /dev/null 2>&1");

    if ($L_result != 0)
      {
	return "NOK" ;
      }
    
    my @L_output = qx{/usr/bin/pkginfo $command -l $L_package} ;
 
    foreach my $L_line (@L_output)
      {
	chomp($L_line) ;
	if ($L_line =~ m/ERROR:/)
	  {
	    return "NOK" ;
	  }
	if ($L_line =~ m/VERSION:[\s\t]*([0-9.]*)/)
	  {
	    $$R_version = $1 ;
	  }
	if ($L_line =~ m/NAME:[\s\t]*(.*)/)
	  {
	    $$R_name = $1 ;
	  }
	if ($L_line =~ m/BASEDIR:[\s\t]*(.*)/)
	  {
	    $$R_basedir = $1 ;

	    # for a shared package:
	    # the base dir is checked only if the package is accessible
	    # via the mounting point (meaning that the base dir has been retrieved)
	    # -> if the base dir is different, the package is not the right one
	    #    we consider it as not installed
	    #
	    # if the access is local (before mounting point has been set)
	    # the base dir was not retrieved so don't check anything !
	    #
	    #
	    if (($L_location eq "SHARED") 
		&& ($G_swdb_remote_access eq "TRUE") 
		&& ($$R_basedir ne $G_swdb_basedir)) {
	      return "NOK" ;
	    }
	  }
      }

    return "OK" ;
      
  }

#--------------------------------------------------------------
#
#  is_feature_installed
#
#    check if at least one package of a feature is installed
#    return TRUE or FALSE
#
#--------------------------------------------------------------

sub is_feature_installed {

  my ($feature) = @_;

  my $name;
  my $location;
  my $version;
  my $basedir;

  foreach my $package (@G_packages) {
    if (@$package[$G_package_feature] eq $feature) {
      $name = @$package[$G_package_name],
      $location = @$package[$G_package_location];
      if (get_package($name, $location, \$version, \$name, \$basedir) eq "OK") {
	return "TRUE";
      }
    }
  }

  return "FALSE";

}

#--------------------------------------------------------------
#
#  check_package
#
#    check if a package is installed and if its version
#    is correct
#
#    1 : name of the package
#    2 : location
#    3 : version to be tested (can be a list if type=EXACT
#    4 : type of comparison
#        EXACT   : must match the exact version
#        AT_LEAST: minimal version
#
#--------------------------------------------------------------

sub check_package
  {
    my ($L_package, $L_location, $L_version, $L_comp, $L_mandatory) = @_ ;

    my $L_status = "OK" ;
    my $L_current_version ;
    my $L_current_name ;
    my $L_basedir ;

    my $return = get_package($L_package, $L_location, 
			     \$L_current_version, \$L_current_name, \$L_basedir) ;
#    if ($return eq NA)
#      {
#	return ;
#      }

    if ($return ne "OK") {
      if ($L_mandatory eq "YES") {
	

#=item package <name> not installed.
#
#    Please install the specified package.
#
#=cut

	LOG::printError("package %s not installed", $L_package) ;
	my $status = "FAILED";
	LOG::printResult($status, "Package %s", $L_package) ;
      } else {
	LOG::printWarning("package %s not installed", $L_package) ;
      }
      return ;
    }	

    my $status = compare_version($L_current_version, $L_version, $L_comp) ;

    if ($status eq "FALSE")
      {

#=item package <name>: version <current> instead of <expected>
#
#    Please install the correct version of the specified package.
#
#=cut

	LOG::printError("package %s: version %s instead of %s",
			$L_package, $L_current_version, $L_version) ;
	$L_status = "FAILED" ;
      }
    else
      {
	if ($status eq "LESS")
	  {

#=item package <name>: version <current> (at least <minimum> required)
#
#    Please install the correct version of the specified package.
#
#=cut

	    LOG::printError("package %s: version %s (at least %s required)", 
			    $L_package, $L_current_version, $L_version) ;
	    $L_status = "FAILED" ;
	  }
      }
    
    LOG::printResult($L_status, "Package %s", $L_package) ;
  }


#--------------------------------------------------------------
#
#  check_package_array
#
#    check if a array of package is installed
#
#    1: package array
#    2: MEN or NMEN
#
#--------------------------------------------------------------

sub check_package_array
  {
    my ($pkgarr, $role) = @_ ;
    
    my $field ;
    
    if ($role eq "MEN") {
      $field = $G_package_men ;
    } elsif ($role eq "DISKLESS") {
      $field = $G_package_diskless ;
    } else {
      $field = $G_package_dataless ;
    }
    
    #
    # check local packages if it makes sense
    #

    LOG::printMsg(0,"") ;
    LOG::printMsg(0,"Local packages") ;
    LOG::printMsg(0,"") ;
    foreach my $pkg (@$pkgarr)
      {
	if ((@$pkg[$field] eq "YES") && (@$pkg[$G_package_location] eq "LOCAL"))
	  {
	    check_package(@$pkg[$G_package_name], 
			  @$pkg[$G_package_location], 
			  @$pkg[$G_package_version],
			  @$pkg[$G_package_comp],
			  @$pkg[$G_package_mandatory]);
	  }
      }

    if ($G_swdb_accessible eq "FALSE") {
      LOG::printMsg(0,"") ;
      LOG::printMsg(0,"Shared packages currently not accessible") ;
      LOG::printMsg(0,"") ;
      return ;
    }

    #
    # check shared packages
    #
    LOG::printMsg(0,"") ;
    LOG::printMsg(0,"Shared packages (on %s)", $G_swdb_dir) ;
    LOG::printMsg(0,"") ;
    foreach my $pkg (@$pkgarr)
      {
	if (@$pkg[$G_package_location] eq "SHARED")
	  {
	    check_package(@$pkg[$G_package_name], 
			  @$pkg[$G_package_location], 
			  @$pkg[$G_package_version],
			  @$pkg[$G_package_comp],
			  @$pkg[$G_package_mandatory]);
	  }
      }
  }

#--------------------------------------------------------------
#
#  load_patch
#
#    load patch list into an array
#
#    1 : array name
#    2 : location
#
#--------------------------------------------------------------

sub load_patch {

  my ($arrayref, $location) = @_;

  my $optcmd = "";

  if ($location eq "SHARED") {
    $$arrayref = \@G_shared_patch;
    if ($G_shared_patch_loaded) {
      return ;
    } else {
      $optcmd = "-R $G_swdb_dir" ;
      $G_shared_patch_loaded = $TRUE;
    }
  } elsif ($location eq "LOCAL") {
    $$arrayref = \@G_local_patch;
    if ($G_local_patch_loaded) {
      return ;
    } else {
      $G_local_patch_loaded = $TRUE;
    }
  } else {
    LOG::printMsg("Invalid location (%s)", $location) ;
    exit 1;
  }

  my @L_result = qx{/usr/sbin/patchadd $optcmd -p | /usr/bin/cut -f2 -d' '} ;
  foreach my $line (@L_result) {
    if (chomp($line) ne "") {
      push @$$arrayref, $line;
    }
  }

}

#--------------------------------------------------------------
#
#  get_patch
#
#    get info about a specified patch
#
#    1 : in: name of the package
#    2 : in: location
#
#    return TRUE if the patch exists
#
#--------------------------------------------------------------

sub get_patch {
    my ($L_patch, $L_location) = @_ ;

    my $arrayref;
    my $optcmd = "" ;

    load_patch(\$arrayref, $L_location);

    my $i = 0;
    for (my $i = 0 ; $i < @{$arrayref} ; $i++) {
      if ($$arrayref[$i] eq $L_patch) {
	return "TRUE";
      }
    }
    
    return "FALSE" ;
  }

#--------------------------------------------------------------
#
#  check_patch
#
#    check if a patch is installed and if its version
#    is correct
#
#    1 : name of the patch
#    2 : location
#    3 : to check or not
#
#--------------------------------------------------------------

sub check_patch
  {
    my ($L_patch, $L_location, $L_tocheck) = @_ ;

    if ($L_tocheck ne "YES") {
      return ;
    }

    my $L_status = "OK" ;

    if (($L_location eq "SHARED") && ($G_swdb_accessible eq "FALSE")) {
      return ;
    }

    if (get_patch($L_patch, $L_location) ne "TRUE")
      {

#=item Patch <patch> not installed
#
#    The specified patch must be installed.
#
#=cut

	LOG::printError("Patch %s not installed", $L_patch) ;
	$L_status = "FAILED";
      }
    LOG::printResult($L_status, "Patch %s", $L_patch) ;
  }

#--------------------------------------------------------------
#
#  check_file
#
#    1 : name of the file to check (exists, readable)
#
#--------------------------------------------------------------

sub check_file
  {
    my ($filename) = @_ ;

    if (! -e $filename)
      {

#=item <file> does not exist
#
#    This may be caused by a wrong pathname.
#
#=cut

	LOG::printError("%s does not exist", $filename) ;
	return "FALSE" ;
     }

    if (! -r $filename)
      {

#=item cannot read <file>: permission denied
#
#    Please check that the root account can access the specified file.
#
#=cut

        LOG::printError("can't read %s: permission denied", $filename) ;
	return "FALSE" ;
      }

    return "TRUE" ;
  }

#--------------------------------------------------------------
#
#  check_service
#
#    check if a service is defined in /etc/services
#
#    1 : name of the service
#    2 : protocol (tcp/udp)
#
#--------------------------------------------------------------

sub check_service
  {
    my ($L_service,$L_protocol) = @_ ;

    my $L_status = "OK" ;
    my $protocol;
    my $wrong_protocol;
    my $save_protocol;
    my $found;

    LOG::printMsg(2,"Check service %s, protocol %s",
		  $L_service, $L_protocol) ;

    #
    # crfs service must be defined in /etc/services
    # a service can appear twice with different protocol
    #
    $L_status = "OK" ;
    if (check_file("/etc/services") eq "TRUE")
      {
	open(FH,"</etc/services") or die "/etc/services: $!";
	$found = "FALSE" ;
	$wrong_protocol = "FALSE" ;
	while (<FH>)
	  {
	    my $line;
	    chomp($line = $_) ;
	    $line =~ s/#.*// ;
	    if ($line =~ m/[\s\t]*([^\s\t]*)[\s\t]*([0-9]*)\/([a-z]*)/)
              {
	        my $service = $1 ;
	        my $port = $2 ;
	        my $protocol = $3 ;
	        if ($service eq $L_service)
	          {
		    if ($protocol eq $L_protocol)
		      {
		        $found = "TRUE" ;
		        $wrong_protocol = "FALSE" ;
		      }
		    else
		      {
		        $wrong_protocol = "TRUE" ;
			$save_protocol = $protocol;
		      }
	          }
	      }
           }

	if ($found eq "TRUE")
	  {
	    $L_status = "OK" ;
	  }
	else
	  {
	  if ($wrong_protocol eq "TRUE")
	    {

#=item <service> service: <current> instead of <expected>
#
#    Please correct the protocol for the specified service
#    in /etc/services.
#
#=cut

	      LOG::printError ("%s service: %s instead of %s", $L_service, $save_protocol, $L_protocol);
	      $L_status = "FAILED" ;
	    }
	  else
	    {

#=item <service> service must be defined in /etc/services
#
#    Please add an entry for the specified service in /etc/services.
#    Otherwise, daemons using an unspecified service might not work
#    correctly.
#
#=cut
 
	      LOG::printError("%s service must be defined in /etc/services", $L_service) ;
	      $L_status = "FAILED" ;
	    }
	}
      }

    close (FH) ;

    LOG::printResult($L_status, "%s defined in /etc/services",
		     $L_service) ;
  }

#--------------------------------------------------------------
#
#  check_installation
#
#      check information related to the Solaris OS
#
#--------------------------------------------------------------

sub check_installation
  {
    my $L_status = "OK" ;

    my $i;

    LOG::printMsg(0, "") ;
    LOG::printMsg(0, "OS and Software checking") ;
    LOG::printMsg(0, "") ;

    my $bits = qx{/usr/bin/isainfo -b} ;
    if ($bits != "64") {
#=item Invalid kernel mode: <bits>-bit instead of 64-bit mode
#
#    Please install the correct Solaris version.
#
#=cut

	LOG::printError("Invalid kernel mode: %s-bit instead of 64-bit mode",
			$bits) ;
	$L_status = "FAILED";
      }
    LOG::printResult($L_status,"64-bit kernel mode" ) ;

    #
    #  check various packages
    #

    determine_swdb ;

    if (guess_node_role eq "DISKFULL") {
      check_package_array(\@G_packages, "MEN") ;
    } elsif (guess_node_role eq "DISKLESS") {
      check_package_array(\@G_packages, "DISKLESS") ;
    } else {
      check_package_array(\@G_packages, "DATALESS") ;
    }

    #
    #  check various patches
    #
    LOG::printMsg(0, "") ;
    LOG::printMsg(0, "Patches") ;
    LOG::printMsg(0, "") ;
    LOG::printMsg(0, "Can take a long time ...") ;
    LOG::printMsg(0, "") ;
    for ($i = 0 ; $i < scalar(@G_patches) ; $i++) {
      if (guess_node_role eq "DISKFULL") {
	check_patch($G_patches[$i][$G_patch_name], 
		    $G_patches[$i][$G_patch_location],
		    $G_patches[$i][$G_patch_men]) ;
      } elsif (guess_node_role eq "DISKLESS") {
	check_patch($G_patches[$i][$G_patch_name], 
		    $G_patches[$i][$G_patch_location],
		    $G_patches[$i][$G_patch_diskless]) ;
      } else {
	check_patch($G_patches[$i][$G_patch_name], 
		    $G_patches[$i][$G_patch_location],
		    $G_patches[$i][$G_patch_dataless]) ;
      }
    }

    #
    # check if shared file for patchabitlity support is present 
    # 
    if ($G_swdb_accessible eq "TRUE") {
      $L_status = "FAILED" ;
      if (-f "$G_swdb_dir/$G_swdb_path_confshare/$G_swdb_file_confshare") {
	$L_status = "OK" ;
      } else {
	
#=File for patchability of shared packages not present.
#
#    Please use the "nhadm confshare" command to create this file.
#
#=cut

	LOG::printError("File for patchability of shared packages not present.") ;
      }
    LOG::printResult($L_status, "INST_RELEASE file for shared packages") ;
    }
  }

#--------------------------------------------------------------
#
#  display_package_array
# 
#      display information regarding package array
#
#--------------------------------------------------------------

sub display_package_array
  {
    my ($pkgarr, $role) = @_ ;

    my $field ;
    my $L_name;
    my $L_location ;
    my $L_version ;
    my $L_basedir ;

    if ($role eq "MEN") {
      $field = $G_package_men ;
    } elsif ($role eq "DISKLESS") {
      $field = $G_package_diskless ;
    } else {
      $field = $G_package_dataless ;
    }

    determine_swdb ;

    #
    # display local packages
    #
    LOG::printMsg(0,"") ;
    LOG::printMsg(0,"Local packages") ;
    LOG::printMsg(0,"") ;
    foreach my $pkg (@$pkgarr)
      {
	if ((@$pkg[$field] eq "YES") && (@$pkg[$G_package_location] eq "LOCAL"))
         {
	   my $L_package = @$pkg[$G_package_name];
	   my $L_location = @$pkg[$G_package_location];
	   my $return = get_package($L_package, $L_location, \$L_version, \$L_name, \$L_basedir) ;
	   if ($return eq "OK")
	     {
	       printLine("    package %s", $L_package) ;
	       printLine("        name:        %s", $L_name) ;
	       printLine("        version:     %s", $L_version) ;
	       if ($L_basedir ne "") {
		 printLine("        basedir:     %s", $L_basedir) ;
	       }
	     }
	   elsif ($return eq "NOK")
	     {
	       printLine("    package %s -> not installed", $L_package) ;
	     }
	}
      }

    if ($G_swdb_accessible eq "FALSE") {
      LOG::printMsg(0,"") ;
      LOG::printMsg(0,"Shared packages currently not accessible") ;
      LOG::printMsg(0,"") ;
      return ;
    }

    #
    # display shared packages
    #
    LOG::printMsg(0,"") ;
    LOG::printMsg(0,"Shared packages (on %s)", $G_swdb_dir) ;
    LOG::printMsg(0,"") ;
    foreach my $pkg (@$pkgarr)
      {
	if (@$pkg[$G_package_location] eq "SHARED")
         {
	   my $L_package = @$pkg[$G_package_name];
	   my $L_location = @$pkg[$G_package_location];
	   my $return = get_package($L_package, $L_location, \$L_version, \$L_name, \$L_basedir) ;
	   if ($return eq "OK")
	     {
	       printLine("    package %s", $L_package) ;
	       printLine("        name:        %s", $L_name) ;
	       printLine("        version:     %s", $L_version) ;
	       if ($L_basedir ne "") {
		 printLine("        basedir:     %s", $L_basedir) ;
	       }
	     }
	   elsif ($return eq "NOK")
	     {
	       printLine("    package %s -> not installed", $L_package) ;
	     }
	}
      }
  }
#--------------------------------------------------------------
#
#  display
#
#      display information regarding OS
#
#--------------------------------------------------------------

sub display
  {

    printLine("") ;
    printLine("System information") ;
    printLine("") ;

    chomp(my $hardware  = qx{/usr/bin/uname -i}) ;
    my $release = qx{/usr/bin/grep "Solaris" /etc/release};
    $release =~ s/^[\s\t]*//;
    $release =~ s/[\s\t]*$//;
    chomp($release);

    printLine("    node name:  %s", $E_node_name) ;
    printLine("    Hardware:   %s", $hardware) ;
    printLine("    OS release: %s", $release) ;

    if (guess_node_role eq "DISKFULL") {
      display_package_array(\@G_packages, "MEN") ;
    } elsif (guess_node_role eq "DISKLESS") {
      display_package_array(\@G_packages, "DISKLESS") ;
    } else {
      display_package_array(\@G_packages, "DATALESS") ;
    }
  }

#--------------------------------------------------------------
#
#  configure_shared
#
#--------------------------------------------------------------

sub configure_shared
  {
    my ($dir) = @_ ;

    if ($dir eq "") {
      #
      # the default directory is used if accessible
      #
      determine_swdb ;
      if ($G_swdb_accessible eq "FALSE") {
	LOG::printError("Directory for shared packages not accessible") ;
	return ;
      }
      $dir = $G_swdb_dir ;
    }

    my $result = qx{/usr/bin/mkdir -p $dir/$G_swdb_path_confshare 2>&1} ;
    if ( $? ne 0 ) {
      LOG::printMsg(0,"Error when executing mkdir") ;
      LOG::printMsg(0,$result) ;
      exit 1 ;
    }

    my $filename = "$dir/$G_swdb_path_confshare/$G_swdb_file_confshare";

    $result = qx{/usr/bin/cp /$G_swdb_path_confshare/$G_swdb_file_confshare $filename 2>&1} ;
    if ( $? ne 0 ) {
      LOG::printMsg(0,"Error when executing copy") ;
      LOG::printMsg(0,$result) ;
      exit 1 ;
    }

    LOG::printMsg(0, "File %s created", $filename) ;
  }

#--------------------------------------------------------------
#
#  main
#
#--------------------------------------------------------------

#=back
#
#=cut

{
}
